%%%%
%%  KatanKtantan_lib
%%%%

%%%%
%% State creation
%%%%
function state=create_state_bin(X) %% Convert hexadecimal message into binary message
  state='';
  for i=1:length(X)
    state=[state dec2bin(hex2dec(X(i)), 4)];
  end
end

function L1=create_L1(s)
  L1=[];
  for i=1:13
    L1=[bin2dec(s(i)) L1];
  end
end

function L2=create_L2(s)
  L2=[];
  for i=14:32
    L2=[bin2dec(s(i)) L2];
  end
end

function K=create_K(s)
  K=[];
  for i=1:length(s)
    K=[bin2dec(s(i)) K];
  end
end

%%%%
%% Encryption
%%%%

function r=Fa(L, x, ir, k)
  r=bitxor(L(x(1)), bitxor(L(x(2)), bitxor(L(x(3))&L(x(4)), bitxor(L(x(5))&ir, k))));
end

function r=Fb(L, x, k)
  r=bitxor(L(x(1)), bitxor(L(x(2)), bitxor(L(x(3))&L(x(4)), bitxor(L(x(5))&L(x(6)), k))));
end

function T=round_count(T_i)
  tmp=bitxor(T_i(8), bitxor(T_i(7), bitxor(T_i(5), T_i(3))));
  T=[tmp T_i(1:7)];
end

function [S1 S2 T]=round_f(L1, L2, T_i, x, y, ka, kb)
  T=round_count(T_i);
  r1=Fa(L1, x, T(8), ka);
  r2=Fb(L2, y, kb);
  S1=[r2 L1(1:end-1)];
  S2=[r1 L2(1:end-1)];
end

%%%%
%% Keyschedule
%%%%

function Kr=keyschedule_Katan(K)
  Kr(1)=bitxor(K(79), bitxor(K(60), bitxor(K(49), K(12))));
  Kr(2)=bitxor(K(80), bitxor(K(61), bitxor(K(50), K(13))));
  Kr=[Kr K(1:78)];
end

function [ka kb]=keyschedule_Ktantan(K, T)
  ind=bin2dec([dec2bin(T(8)) dec2bin(T(7)) dec2bin(T(6)) dec2bin(T(5))])+1;
  a=[];
  for i=1:5
    a(i)=K(ind+16*(i-1));
  end
  %% ka
  ka = bitxor(!T(4)&!T(3)&a(1), (T(4)|T(3))&a(bin2dec([dec2bin(T(2)) dec2bin(T(1))])+2));
  %% kb
  kb = bitxor(!T(4)&T(3)&a(5), (T(4)|!T(3))&a(bin2dec([dec2bin(!T(2)) dec2bin(!T(1))])+1));
end

%%%%
%% Decryption
%%%%
function r=Fa_d(L, i, x, ir, k)
  r=bitxor(i, bitxor(L(x(2)+1), bitxor(L(x(3)+1)&L(x(4)+1), bitxor(L(x(5)+1)&ir, k))));
end

function r=Fb_d(L, i, x, k)
  r=bitxor(i, bitxor(L(x(2)+1), bitxor(L(x(3)+1)&L(x(4)+1), bitxor(L(x(5)+1)&L(x(6)+1), k))));
end

function T=round_count_d(T_i)
  tmp=bitxor(T_i(1), bitxor(T_i(8), bitxor(T_i(6), T_i(4))));
  T=[T_i(2:8) tmp];
end

function [S1 S2 T]=round_f_d(L1, L2, T_i, x, y, ka, kb)
  T=round_count_d(T_i);
  r1=Fa_d(L1, L2(1), x, T(8), ka);
  r2=Fb_d(L2, L1(1), y, kb);
  S1=[L1(2:end) r1];
  S2=[L2(2:end) r2];
end

%%%%
%% Keyschedule
%%%%

function Kr=keyschedule_Katan_d(K)
  Kr(79)=bitxor(K(1), bitxor(K(62), bitxor(K(51), K(14))));
  Kr(80)=bitxor(K(2), bitxor(K(63), bitxor(K(52), K(15))));
  Kr=[K(3:80) Kr(79) Kr(80)];
end

%%%%
%%  extract cipher
%%%%
function C=extract_C(L1,L2)
  tmp='';
  C='';
  for i=1:length(L2)
    tmp=[dec2bin(L2(i)) tmp];
  end
  for i=1:length(L1)
    tmp=[dec2bin(L1(i)) tmp];
  end
  for i=1:4:length(tmp)-3
    C=[C dec2hex(bin2dec(tmp(i:i+3)))];
  end
end

%%%%
%% Ktantan
%%%%
function C=Ktantan_enc(MSG, Key) %% Encryption for Ktantan
  %% Initialization
  m=create_state_bin(MSG);
  K_b=create_state_bin(Key);
  L1=create_L1(m);
  L2=create_L2(m);
  K=create_K(K_b);
  clear m;
  clear K_b;
  %% Parameters (x, y)
  x=[13 8 9 6 4];
  y=[19 8 13 11 9 4];
  T=[1 1 1 1 1 1 1 1];
  %% Encryption
  for i=0:253
    [ka kb]=keyschedule_Ktantan(K, T);
    [L1 L2 T]=round_f(L1, L2, T, x, y, ka, kb);
  end
  %% Cipher extraction
  C=extract_C(L1,L2);
end

function M=Ktantan_dec(C, Key) %% Decryption for Ktantan
  c=create_state_bin(C);
  K_b=create_state_bin(Key);
  L1=create_L1(c);
  L2=create_L2(c);
  K=create_K(K_b);
  clear m;
  clear K_b;
  %% Parameters (x, y)
  x=[13 8 9 6 4];
  y=[19 8 13 11 9 4];
  T=[1 1 1 1 1 1 1 1];
  %% Decryption
  for i=253:-1:0
    [ka kb]=keyschedule_Ktantan(K, T);
    [L1 L2 T]=round_f_d(L1, L2, T, x, y, ka, kb);
  end
  %% Message extraction
  M=extract_C(L1,L2);
end